/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.collision;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.model.BlackboardDataFacet;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.model.sinfo.Collision;
import org.dive4elements.river.model.sinfo.CollisionAggregateValue;

/**
 * Facet for a river bed collision value series loaded from the database
 *
 * @author Matthias Schäfer
 */
public class CollisionFacet extends BlackboardDataFacet implements FacetTypes {

    private static final long serialVersionUID = 1;

    public CollisionFacet(final String name, final String description, final String yAxisLabel) {
        super(0, name, description);

        this.metaData.put("X", "chart.longitudinal.section.xaxis.label");
        this.metaData.put("Y", yAxisLabel);
    }

    /**
     * Returns the data this facet requires.
     *
     * @param artifact
     *            the owner artifact.
     * @param context
     *            the CallContext (ignored).
     *
     * @return
     *         the data as CollisionQueryCalculationResult
     */
    @Override
    public Object getData(final Artifact artifact, final CallContext context) {

        final CollisionDbAccess access = new CollisionDbAccess((D4EArtifact) artifact);
        final Collision series = Collision.getSeries(access.getId());
        final List<CollisionAggregateValue> values = CollisionAggregateValue.getValuesByKm(series, access.getFrom(true), access.getTo(true));
        final Collection<ResultRow> rows = new ArrayList<>();
        for (final CollisionAggregateValue value : values) {
            rows.add(ResultRow.create().putValue(GeneralResultType.station, value.getStation()) //
                    .putValue(SInfoResultType.collisionCount, value.getCount())
                    .putValue(SInfoResultType.collisionGaugeW, value.getGaugeW()));
        }
        return new CollisionQueryCalculationResult(series.getFilename(), rows);
    }

    /**
     * Create a deep copy of this Facet.
     *
     * @return a deep copy.
     */
    @Override
    public CollisionFacet deepCopy() {
        final CollisionFacet copy = new CollisionFacet(this.name, this.description, this.metaData.get("Y"));
        copy.set(this);
        return copy;
    }
}